package top.dreamcenter.go.client;

import top.dreamcenter.go.msg.Cross;
import top.dreamcenter.go.msg.NewClick;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

public class MainFrame extends JFrame {

    private static final int BASE_HEIGHT = 26;
    private static final int BASE_FROM = 20;

    Cross[][] map = new Cross[9][9];
    Cross now = Cross.NONE;

    public MainFrame(ObjectInputStream oi, ObjectOutputStream os) {
        now = (Cross) Client.recMsg(oi);
        System.out.println(now);
        setTitle(now == Cross.BLACK ? "●" : "○");

        setSize(200,200 + BASE_HEIGHT);

        addMouseListener(new MouseAdapter() {
            @Override
            public void mouseReleased(MouseEvent e) {
                int x = e.getX() - BASE_FROM;
                int y = e.getY() - BASE_FROM - BASE_HEIGHT;

                int col = (x%BASE_FROM < BASE_FROM / 2) ? (x / BASE_FROM) : (x / BASE_FROM + 1);
                int row = (y%BASE_FROM < BASE_FROM / 2) ? (y / BASE_FROM) : (y / BASE_FROM + 1);
                System.out.printf("[click] row: %d; col: %d;\n", row, col);

                NewClick newClick = NewClick.newInstance(now, row + 1, col + 1);

                Client.sendMsg(os, newClick);

                repaint();
            }
        });

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setVisible(true);

        new Thread(() -> {
            while (true) {
                try {
                    String o = (String) Client.recMsg(oi);
                    map = DataUtil.rawData(o);
                    repaint();
                } catch (Exception e){
                    e.printStackTrace();
                    System.out.println("连接断开");
                    break;
                }
            }
        }).start();
    }

    @Override
    public void paint(Graphics g) {
        g.setColor(Color.white);
        g.fillRect(0,BASE_HEIGHT,200,200);

        g.setColor(Color.lightGray);
        for (int i = 1; i <= 9; i++) {
            drawLine(g, BASE_FROM, BASE_FROM * i, BASE_FROM * 9, BASE_FROM * i);
            drawLine(g, BASE_FROM * i, BASE_FROM, BASE_FROM * i, BASE_FROM * 9);
        }

        g.setColor(Color.black);

        for (int i = 0; i < map.length; i++) {
            for (int j = 0; j < map.length; j++) {
                if (map[i][j] != Cross.NONE) {
                    drawDot(g, i+1, j+1, map[i][j]);
                }
            }
        }
    }

    @Override
    public void update(Graphics g) {
        Image tmp = createImage(200, 200);
        Graphics tmpG = tmp.getGraphics();
        paint(tmpG);
        g.drawImage(tmp, 0,BASE_HEIGHT,null);
    }

    /**
     * 绘制线
     * @param g g
     * @param x1 x1
     * @param y1 y1
     * @param x2 x2
     * @param y2 y2
     */
    private void drawLine(Graphics g, int x1, int y1, int x2, int y2) {
        g.drawLine(x1, BASE_HEIGHT + y1, x2, BASE_HEIGHT + y2);
    }

    /**
     *绘制实心圆
     * @param g g
     * @param cx 圆中心点x
     * @param cy 圆中心点y
     * @param r 半径
     */
    private void fillOval(Graphics g, int cx, int cy, int r) {
        g.fillOval(cx - r, cy - r, r<<1, r<<1);
    }

    /**
     * 绘制空心圆
     * @param g g
     * @param cx 圆中心点x
     * @param cy 圆中心点y
     * @param r 半径
     */
    private void drawOval(Graphics g, int cx, int cy, int r) {
        g.drawOval(cx - r, cy - r, r<<1, r<<1);
    }

    /**
     * 绘制棋子
     * @param g g
     * @param row 行
     * @param col 列
     */
    private void drawDot(Graphics g, int row, int col, Cross cross){
        if (cross == Cross.BLACK) {
            fillOval(g,  col * BASE_FROM , row * BASE_FROM + BASE_HEIGHT, BASE_FROM / 2);
        }
        if (cross == Cross.WHITE) {
            drawOval(g,  col * BASE_FROM , row * BASE_FROM + BASE_HEIGHT, BASE_FROM / 2 - 1);
            drawOval(g,  col * BASE_FROM , row * BASE_FROM + BASE_HEIGHT, BASE_FROM / 2 - 2);
        }

        if (cross == Cross.BLACK_CUR) {
            fillOval(g,  col * BASE_FROM , row * BASE_FROM + BASE_HEIGHT, BASE_FROM / 2);
            drawPlus(g, col * BASE_FROM , row * BASE_FROM + BASE_HEIGHT, BASE_FROM>>2, Color.WHITE);
        }
        if (cross == Cross.WHITE_CUR) {
            drawOval(g,  col * BASE_FROM , row * BASE_FROM + BASE_HEIGHT, BASE_FROM / 2 - 1);
            drawOval(g,  col * BASE_FROM , row * BASE_FROM + BASE_HEIGHT, BASE_FROM / 2 - 2);
            drawPlus(g, col * BASE_FROM , row * BASE_FROM + BASE_HEIGHT, BASE_FROM>>2, Color.BLACK);
        }
    }

    private void drawPlus(Graphics g, int cx, int cy, int r, Color color){
        g.setColor(color);
        g.fillRect(cx- r, cy - 1, r * 2 + 1, 3);
        g.fillRect(cx - 1, cy - r, 3, r * 2 + 1);
        g.setColor(Color.BLACK);
    }

}
