package top.dreamcenter.go.server;

import top.dreamcenter.go.msg.Cross;
import top.dreamcenter.go.msg.KVPair;
import top.dreamcenter.go.msg.NewClick;

import java.util.ArrayList;
import java.util.List;

public class CalDataUtil {

    public static Cross[][] getInitMap() {
        Cross[][] map = new Cross[9][9];
        for (int i = 0; i < map.length; i++) {
            for (int j = 0; j < map.length; j++) {
                map[i][j] = Cross.NONE;
            }
        }
        return map;
    }

    public static boolean draw(NewClick newClick, Cross[][] map) {
        Cross cross = newClick.color;
        int row = newClick.pos.k;
        int col = newClick.pos.v;
        return draw(cross, row, col, map);
    }

    public static boolean draw(Cross cross, int row, int col, Cross[][] map) {
        int width = map.length;
        if (row > width || row < 1 || col > width || col < 1 || (map[row - 1][col - 1] != Cross.NONE && cross != Cross.NONE)) return false;
        map[row - 1][col - 1] = cross;
        if (cross != Cross.NONE) check(cross, map);
        return true;
    }

    public static void printMap(Cross[][] map, Cross now){
        System.out.println();
        int width = map.length - 1;
        System.out.println("O A B C D E F G H I");
        for (int i = 0; i < map.length; i++) {
            System.out.print(i + 1 + " ");
            for (int j = 0; j < map.length; j++) {
                if (map[i][j] == Cross.NONE){
                    String tmp = "┼-";
                    if (i == 0) tmp = "┬-";
                    if (i == width) tmp = "┴-";
                    if (j == 0) tmp = "├-";
                    if (j == width) tmp = "┤ ";

                    if (i == 0 && j == 0) tmp = "┌-";
                    if (i == 0 && j == width) tmp = "┐";
                    if (i == width && j == 0) tmp = "└-";
                    if (i == width && j == width) tmp = "┘";

                    System.out.print(tmp);
                } else if (map[i][j] == Cross.BLACK){
                    System.out.print("● ");
                } else {
                    System.out.print("○ ");
                }
            }
            System.out.println();
        }
        System.out.print((now == Cross.BLACK ? " [ ● ] " : " [ ○ ] ") + " : ");
    }

    /**
     * clear dead : BFS
     * @param cross cur
     * @param map map
     */
    private static void check (Cross cross, Cross[][] map) {
        int width = map.length - 1;
        // 遍历 黑色，出现黑色时，查找边缘，只要存在存在空白则安全。bfs查找

        Cross obc = cross == Cross.BLACK ? Cross.WHITE : Cross.BLACK;

        boolean[][] aliveConfirmed = new boolean[map.length][map.length]; // 已被确认为活棋的区域

        for (int i = 0; i < map.length; i++) {
            for (int j = 0; j < map.length; j++) {
                // 为目标色，并且该位置未确认活棋
                if (map[i][j] == obc && !aliveConfirmed[i][j]) {
                    // BFS 查找是否存在 NONE 边界
                    int len = 0;
                    List<KVPair> queue = new ArrayList<>();
                    queue.add(new KVPair(i, j));
                    List<KVPair> colored = new ArrayList<>();
                    colored.add(new KVPair(i, j));

                    boolean isDead = true;
                    while (len < queue.size()) {
                        // 获取当前的四周，插入到queue
                        KVPair kvPair = queue.get(len);
                        int row = kvPair.k;
                        int col = kvPair.v;

                        // 上
                        if (kvPair.k != 0 && isAliveOrAdd(colored, map, row - 1, col, obc, queue)) {
                            isDead = false;
                            break;
                        }

                        // 下
                        if (kvPair.k != width && isAliveOrAdd(colored, map, row + 1, col, obc, queue)) {
                            isDead = false;
                            break;
                        }

                        // 左
                        if (kvPair.v != 0 && isAliveOrAdd(colored, map, row, col - 1, obc, queue)) {
                            isDead = false;
                            break;
                        }

                        // 右
                        if (kvPair.v != width && isAliveOrAdd(colored, map, row, col + 1, obc, queue)) {
                            isDead = false;
                            break;
                        }

                        len++;
                    }

                    if (isDead) {
                        // 去除操作
                        colored.forEach(item -> draw(Cross.NONE, item.k + 1, item.v + 1, map));
                    } else {
                        // 确认活棋
                        colored.forEach(item -> aliveConfirmed[item.k][item.v] = true);
                    }
                }
            }
        }

    }

    private static boolean isAliveOrAdd(List<KVPair> colored, Cross[][] map, int row, int col, Cross obc, List<KVPair> queue){
        if (isColored(colored, row, col)) return false;

        // 活的，则退出整个大循环
        if (map[row][col] == Cross.NONE) {
            return true;
        }

        // 为己方的，则加入队列，否则对方不进行操作
        if (map[row][col] == obc) {
            queue.add(new KVPair(row, col));
            colored.add(new KVPair(row, col));
        }
        return false;
    }

    private static boolean isColored(List<KVPair> colored, int row, int col) {
        for (KVPair tmp:colored) {
            if (tmp.k == row && tmp.v == col) return true;
        }
        return false;
    }
}
